import React, { PropTypes } from 'react'
import Markdown from 'react-remarkable'
import styled, { css } from 'styled-components'
import Box from './Box'
import TextStyles from './../utils/TextStyles'

const getMaxLineWidth = (maxLineWidth) => {
  if (typeof maxLineWidth === 'boolean') {
    if (maxLineWidth) return `max-width: 40em;`
    return ''
  }
  return `max-width: ${maxLineWidth};`
}

const styles = ({ indentLists, theme, maxLineWidth }) => css`
  a {
    color: ${theme.colors.primary}
  }

  h1 {
    ${TextStyles.display(1, theme)}
    margin-top: 1em;
    margin-bottom: 1em;
  }

  h2 {
    ${TextStyles.display(2, theme)}
    margin-top: 1em;
    margin-bottom: 1em;
  }

  h3 {
    ${TextStyles.display(3, theme)}
    margin-top: 1em;
    margin-bottom: 1em;
  }

  h4, h5, h6 {
    ${TextStyles.display(4, theme)}
    margin-top: 1em;
    margin-bottom: 1em;
  }

  p, ul, ol {
    ${TextStyles.body(1, theme)}
    ${getMaxLineWidth(maxLineWidth)}
    margin: 1em 0;
  }

  ${indentLists ? `
    ul,
    ol {
      margin-left: 1.15em;
    }
  ` : ''}

  ul {
    list-style: disc;
    list-style-position: outside;
  }

  ol {
    list-style: decimal;
  }

  em {
    font-style: italic;
  }

  strong {
    font-weight: 500;
  }

  p > code,
  li > code,
  table code {
    ${TextStyles.body(1, theme)}
    display: inline-block;
    background-color: ${theme.colors.tint[0]};
    box-shadow: inset 0 0 0 1px ${theme.colors.tint[1]};
    padding: 0 0.5em;
    border-radius: ${theme.borderRadius[0]};

    &:before,
    &:after {
      content: '\\00a0';
      letter-spacing: -0.2em;
    }
  }

  pre {
    margin: 1em 0;
    overflow-x: auto;
    text-align: left;
    display: block;
    white-space: pre;
    background-color: ${theme.colors.tint[0]};
    box-shadow: inset 0 0 0 1px ${theme.colors.tint[1]};
    padding: ${theme.space[3]};
    border-radius: ${theme.borderRadius[0]};

    & code {
      ${TextStyles.mono(1, theme)}
      padding: 0;
      background: none;

      &:before,
      &:after {
        content: none;
        letter-spacing: 0;
      }
    }
  }

  hr {
    background: ${theme.colors.tint[1]};
    border: 0;1
    height: 1px;
    margin: 30px 0;
  }

  table {
    width: 100%;
    margin: 1em 0;
  }

  thead {
    border-bottom: 1px solid ${theme.colors.tint[1]};
    background-color: ${theme.colors.tint[0]};
    position: relative;
    text-align: left;
    padding: 0 20px;
    box-sizing: content-box;
  }

  tbody tr {
    border-bottom: 1px solid ${theme.colors.tint[1]};
    background: ${theme.colors.white[0]};
  }

  td, th {
    padding: 0.8em;
  }
`

const parseProps = ({ children, ...props }) => {
  if (typeof children === 'string') {
    return { children: React.createElement(Markdown, { source: children }) , ...props }
  }
  return { children, ...props }
}

const Content = styled(({ markdown, indentLists, maxLineWidth, ...props }) =>
  React.createElement(Box, parseProps(props)))`${styles}`

Content.propTypes = {
  ...Box.propTypes,
  markdown: PropTypes.bool,
  indentLists: PropTypes.bool,
  maxLineWidth: PropTypes.oneOfType([
    PropTypes.bool,
    PropTypes.string
  ])
}

Content.defaultProps = {
  indentLists: true,
  maxLineWidth: true
}

Content.displayName = 'Content'

export default Content
